@@ -2,6 +2,7 @@ using System;
 using System.Collections.Generic;
 using System.Text;
 using Mono.Unix;
+using Gtk;
 
 namespace Tomboy
 {
@@ -25,7 +26,8 @@ namespace Tomboy
 		Gtk.HPaned hpaned;
 		Gtk.VBox content_vbox;
 		Gtk.TreeViewColumn matches_column;
-
+		Gtk.HBox no_matches_box;	//to display a message with buttons when no results are found.
+		
 		Notebooks.NotebooksTreeView notebooksTree;
 
 		// Use the following like a Set
@@ -159,15 +161,6 @@ namespace Tomboy
 			matches_window = new Gtk.ScrolledWindow ();
 			matches_window.ShadowType = Gtk.ShadowType.In;
 
-			// Reign in the window size if there are notes with long
-			// names, or a lot of notes...
-			Gtk.Requisition tree_req = tree.SizeRequest ();
-			if (tree_req.Height > 420)
-				matches_window.HeightRequest = 420;
-
-			if (tree_req.Width > 480)
-				matches_window.WidthRequest = 480;
-
 			matches_window.HscrollbarPolicy = Gtk.PolicyType.Automatic;
 			matches_window.VscrollbarPolicy = Gtk.PolicyType.Automatic;
 			matches_window.Add (tree);
@@ -355,6 +348,7 @@ namespace Tomboy
 
 			Gtk.TreeViewColumn title = new Gtk.TreeViewColumn ();
 			title.Title = Catalog.GetString ("Note");
+			title.MinWidth = 150; // Fix for bgo 575337 - "Matches" column causes notes name not to be shown. jjennings jul 13, 2011
 			title.Sizing = Gtk.TreeViewColumnSizing.Autosize;
 			title.Expand = true;
 			title.Resizable = true;
@@ -491,14 +485,22 @@ namespace Tomboy
 
 			IDictionary<Note,int> results =
 				search.SearchNotes(text, false, selected_notebook);
-			foreach (Note note in results.Keys){
-				current_matches.Add(note.Uri, results[note]);
+			
+			// if no results found in current notebook ask user whether
+			// to search in all notebooks
+			if (results.Count == 0 && selected_notebook != null) {
+				NoMatchesFoundAction ();
 			}
+			else {
+				foreach (Note note in results.Keys){
+					current_matches.Add (note.Uri, results[note]);
+				}
 
-			AddMatchesColumn ();
-			store_filter.Refilter ();
-			tree.ScrollToPoint (0, 0);
-			UpdateMatchNoteCount (current_matches.Count);
+				AddMatchesColumn ();
+				store_filter.Refilter ();
+				tree.ScrollToPoint (0, 0);
+				UpdateMatchNoteCount (current_matches.Count);
+			}
 		}
 
 		void AddMatchesColumn ()
@@ -593,6 +595,51 @@ namespace Tomboy
 			status_bar.Pop (0);
 			status_bar.Push (0, status);
 		}
+		
+		// called when no search results are found in the selected notebook
+		void NoMatchesFoundAction ()
+		{
+			hpaned.Remove (matches_window);
+			String message = Catalog.GetString ("No results found " +
+				"in the selected notebook.\nClick here to " +
+				"search across all notes.");
+			Gtk.LinkButton link_button = new Gtk.LinkButton ("", message);
+			Gtk.LinkButton.SetUriHook(ShowAllSearchResults);
+			link_button.TooltipText = Catalog.GetString 
+				("Click here to search across all notebooks");
+			link_button.Show();
+			Gtk.Table no_matches_found_table = new Gtk.Table (1, 3, false);
+			no_matches_found_table.Attach (link_button, 1, 2, 0, 1,
+			                               Gtk.AttachOptions.Fill | Gtk.AttachOptions.Shrink,
+			                 Gtk.AttachOptions.Shrink,
+			                0, 0
+			              );
+			
+			no_matches_found_table.ColumnSpacing = 4;
+			no_matches_found_table.ShowAll ();
+			no_matches_box = new HBox (false, 0);
+			no_matches_box.PackStart (no_matches_found_table, true, true, 0);
+			no_matches_box.Show ();
+			hpaned.Add2 (no_matches_box);
+		}
+		
+		void RestoreMatchesWindow ()
+		{
+			if (no_matches_box != null) {
+				hpaned.Remove (no_matches_box);
+				hpaned.Add2 (matches_window);
+				no_matches_box = null;
+				RestorePosition();
+			}	
+		}
+		
+		private void ShowAllSearchResults (Gtk.LinkButton button, String param)
+		{
+			TreeIter iter;
+			notebooksTree.Model.GetIterFirst (out iter);
+			notebooksTree.Selection.SelectIter (iter);
+		}		
+		
 
 		/// <summary>
 		/// Filter out notes based on the current search string
@@ -681,17 +728,20 @@ namespace Tomboy
 
 		void OnNotesChanged (object sender, Note changed)
 		{
+			RestoreMatchesWindow ();
 			UpdateResults ();
 		}
 
 		void OnNoteRenamed (Note note, string old_title)
 		{
+			RestoreMatchesWindow ();
 			UpdateResults ();
 		}
 
 		void OnNoteSaved (Note note)
 		{
 			var rect = tree.VisibleRect;
+			RestoreMatchesWindow ();
 			UpdateResults ();
 			tree.ScrollToPoint (rect.X, rect.Y);
 		}
@@ -1221,6 +1271,8 @@ namespace Tomboy
 				entry_changed_timeout.Reset (500);
 				clear_search_button.Sensitive = true;
 			}
+			
+			RestoreMatchesWindow ();
 		}
 
 		// Called in after .5 seconds of typing inactivity, or on
@@ -1297,13 +1349,16 @@ namespace Tomboy
 			if (notebook is Notebooks.SpecialNotebook) {
 				// Bold the "Special" Notebooks
 				crt.Markup = string.Format ("<span weight=\"bold\">{0}</span>", notebook.Name);
+				crt.Editable = false;
 			} else {
 				crt.Text = notebook.Name;
+				crt.Editable = true;
 			}
 		}
 
 		private void OnNotebookSelectionChanged (object sender, EventArgs args)
 		{
+			RestoreMatchesWindow ();
 			Notebooks.Notebook notebook = GetSelectedNotebook ();
 			if (notebook == null) {
 				// Clear out the currently selected tags so that no notebook is selected
@@ -1365,7 +1420,9 @@ namespace Tomboy
 			note = manager.Create ();
 			if (templateNote != null) {
 				// Use the body from the template note
-				string xmlContent = templateNote.XmlContent.Replace (templateNote.Title, note.Title);
+				string xmlContent = templateNote.XmlContent.Replace (XmlEncoder.Encode (templateNote.Title),
+					XmlEncoder.Encode (note.Title));
+				xmlContent = NoteManager.SanitizeXmlContent (xmlContent);
 				note.XmlContent = xmlContent;
 			}
 
@@ -1473,11 +1530,13 @@ namespace Tomboy
 
 		private void OnNoteAddedToNotebook (Note note, Notebooks.Notebook notebook)
 		{
+			RestoreMatchesWindow ();
 			UpdateResults ();
 		}
 
 		private void OnNoteRemovedFromNotebook (Note note, Notebooks.Notebook notebook)
 		{
+			RestoreMatchesWindow ();
 			UpdateResults ();
 		}
 
@@ -1509,15 +1568,18 @@ namespace Tomboy
 			int y;
 			int width;
 			int height;
+			int mon;
 
 			GetPosition(out x, out y);
 			GetSize(out width, out height);
+			mon = Screen.GetMonitorAtPoint(x, y);
 			
 			Preferences.Set (Preferences.SEARCH_WINDOW_X_POS, x);
 			Preferences.Set (Preferences.SEARCH_WINDOW_Y_POS, y);
 			Preferences.Set (Preferences.SEARCH_WINDOW_WIDTH, width);
 			Preferences.Set (Preferences.SEARCH_WINDOW_HEIGHT, height);
 			Preferences.Set (Preferences.SEARCH_WINDOW_SPLITTER_POS, hpaned.Position);
+			Preferences.Set (Preferences.SEARCH_WINDOW_MONITOR_NUM, mon);
 		}
 
 		private void RestorePosition ()
@@ -1527,23 +1589,67 @@ namespace Tomboy
 			object width = Preferences.Get (Preferences.SEARCH_WINDOW_WIDTH);
 			object height = Preferences.Get (Preferences.SEARCH_WINDOW_HEIGHT);
 			object splitter_pos = Preferences.Get (Preferences.SEARCH_WINDOW_SPLITTER_POS);
+			object mon = Preferences.Get (Preferences.SEARCH_WINDOW_MONITOR_NUM);
+			int new_mon, new_x, new_y;
 
 			if (x == null || !(x is int)
 				|| y == null || !(y is int)
 				|| width == null || !(width is int)
 				|| height == null || !(height is int)
-				|| splitter_pos == null || !(splitter_pos is int))
+				|| splitter_pos == null || !(splitter_pos is int)
+				|| mon == null || !(mon is int))
 			return;
-		
+
+			new_mon = Screen.GetMonitorAtPoint ((int) x, (int) y);
+			Logger.Info ("Monitor number returned by GetMonitorAtPoint (actual) is: {0}", new_mon);
+			Logger.Info ("Saved monitor number is: {0}", mon);
+			Logger.Info ("Saved Search window position is {0} x {1}", (int) x, (int) y);
+
+			// If saved monitor number doesn't match the one returned by GetMonitorAtPoint for saved coords
+			// then it means that something has changed in the monitors layout and saved coordinates may not be valid.
+			// Therefore we'll restore the window to the center of the monitor closest to the saved coordinates.
+			/// It will be returned by the same GetMonitorAtPoint call.
+			if (new_mon == (int) mon) {
+				Logger.Info ("Saved monitor number does match the actual one - restoring as-is");
+				new_x = (int) x;
+				new_y = (int) y;
+			} else {
+				Logger.Info ("Saved monitor number does NOT match the actual one - restoring to the center");
+				//getting the monitor size to calculate the center
+				Gdk.Rectangle new_mon_geom = Screen.GetMonitorGeometry (new_mon);
+				new_x = new_mon_geom.Right/2 - (int) width/2;
+				new_y = new_mon_geom.Bottom/2 - (int) height/2;
+			}
+
+			Logger.Info ("Restoring Search window to position {0} x {1} at monitor {2}", new_x, new_y, new_mon);
 			DefaultSize =
 				new Gdk.Size ((int) width, (int) height);
-			Move ((int) x, (int) y);
+			Move (new_x, new_y);
 			hpaned.Position = (int) splitter_pos;
+
 		}
 
 		private void OnExitingEvent (object sender, EventArgs args)
 		{
 			SavePosition ();
 		}
+
+		// This one presents a List of notes that Search has found
+		public List<Note> GetFilteredNotes ()
+		{
+			Gtk.TreeIter iter;
+			List<Note> filtered_notes = new List<Note> ();
+
+			if (store_sort.IterChildren (out iter) == false)
+				return filtered_notes; /* if nothing was found we return empty list */
+
+			// Getting filtered out notes (found by search) to our list
+			// 3 is a column where note itself is stored
+			do {
+				filtered_notes.Add ((Note) store_sort.GetValue (iter, 3));
+			} while (store_sort.IterNext (ref iter));
+
+			return filtered_notes;
+		}
 	}
 }
