@@ -1,5 +1,6 @@
 
 using System;
+using System.Collections.Generic;
 using System.IO;
 using System.Xml;
 using Mono.Unix;
@@ -87,6 +88,7 @@ namespace Tomboy
 			// Create the default note manager instance.
 			string note_path = GetNotePath (cmd_line.NotePath);
 			manager = new NoteManager (note_path);
+			manager.CommandLine = cmd_line;
 
 			SetupGlobalActions ();
 			ActionManager am = Tomboy.ActionManager;
@@ -490,6 +492,8 @@ namespace Tomboy
 		}
 	}
 
+	public delegate void AddinCommandLineEventHandler (Object sender, EventArgs e);
+
 	public class TomboyCommandLine
 	{
 		bool debug;
@@ -504,12 +508,27 @@ namespace Tomboy
 		string note_path;
 		string search_text;
 		bool open_search;
+		bool addin_args;
+
+		List<string> addin_argslist = new List<string> ();
+		public event AddinCommandLineEventHandler AddinCmdLineArgsDetected;
 
 		public TomboyCommandLine (string [] args)
 		{
 			Parse (args);
 		}
 
+		/// <summary>
+		/// Returns all the command line arguments that are
+		/// prefixed with "--addin:" and their parameters.
+		/// </summary>
+		public List<string> Addin_argslist
+		{
+			get {
+				return addin_argslist;
+			}
+		}
+
 		// TODO: Document this option
 		public bool Debug
 		{
@@ -537,7 +556,8 @@ namespace Tomboy
 				open_note_uri != null ||
 				open_search ||
 				open_start_here ||
-				open_external_note_path != null;
+				open_external_note_path != null ||
+				addin_args;
 			}
 		}
 
@@ -710,6 +730,10 @@ namespace Tomboy
 					break;
 
 				default:
+					if (args[idx].StartsWith ("--addin:")) addin_args = true;
+					//All nonrecognized args are added to the add-in argslist in
+					//case they're params for the add-in. (The order is preserved.)
+					addin_argslist.Add (args[idx]);
 					break;
 				}
 
@@ -812,6 +836,8 @@ namespace Tomboy
 				else
 					remote.DisplaySearch ();
 			}
+
+			if (addin_args) AddinCmdLineArgsDetected (this, new EventArgs ());
 		}
 	}
 }
